require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'
require 'csv'

module StudioGame

	class Game
		attr_reader :title
		
		def initialize(title)
			@title = title	
			@players =[]
		end

		def add_player(a_player)
			@players << a_player
		end

		def load_players(from_file)
			CSV.foreach(from_file) do |row|
				 player = Player.new(row[0], row[1].to_i)
				 add_player(player) 
			end
		end

		def save_high_scores(to_file="high_scores.txt")
			File.open(to_file, "w") do |file| 
			 	file.puts "#{@title} High Scores:"
			 	@players.sort.each do |player|
			    file.puts high_score_entry(player)
			    end
			end
		end

		def high_score_entry(player)
			formatted_name = player.name.ljust(50,'.')
			"#{formatted_name} #{player.score}"
		end

		

		def print_name_and_health(player)
			puts "#{player.name.ljust(50,'.')} #{player.health}"
		end

		def print_stats
			 puts "\n #{@title} Statistics" 

			 strong_players,wimpy_players=@players.partition {|strong| strong.strong?}	

			 puts "\n#{strong_players.size} strong players:"
			 strong_players.each do |player| 
			 	print_name_and_health(player)
			 end
			 
			 puts "\n#{wimpy_players.size} wimpy players:"
			 wimpy_players.each do |player| 
			 	print_name_and_health(player)
			 end

			 puts "\nPoints from Treasure by Player:"
			 @players.each do |player|
			 	puts "\n#{player.name} point totals:" 
			 	player.each_found_treasure do |treasure|
	  				puts "#{treasure.points} total #{treasure.name} points"
				end
			 	puts "#{player.points} grand total points"
			 end
			 
			 puts "\nGrand Total Treasure Points:#{total_points}"
			 puts "\n#{@title} High Scores:"
			 @players.sort.each do |player|
			    puts high_score_entry(player)
			 end

		end

		def total_points
		 	@players.each.reduce(0) do |sum,player|
		 		sum + player.points
		 	end
		end

		def play(rounds)
			puts "There are #{@players.size} players in #{@title}"

			@players.each do |player|
		  		puts player
		  	end

			treasures = TreasureTrove::TREASURES
		  	puts "\nThere are #{treasures.size} treasures to be found"
			treasures.each do |treasure|
	           puts "A #{treasure.name} is worth #{treasure.points} points"
			end	  	
	      
			1.upto(rounds) do |round|
			  puts "\nRound #{round}:"
	      	  @players.each do |player|
	        	GameTurn.take_turn(player)
	  	  	  end
	  	   	end
		end
	end
end